@@ -1,4 +1,4 @@
-/* Eye Of Gnome - Image Private Data 
+/* Eye Of Gnome - Image Private Data
  *
  * Copyright (C) 2007 The Free Software Foundation
  *
@@ -27,25 +27,28 @@
 G_BEGIN_DECLS
 
 struct _EogImagePrivate {
-	GnomeVFSURI      *uri;
+	GFile            *file;
 
 	EogImageStatus    status;
+	EogImageStatus    prev_status;
+	gboolean          is_monitored;
 
 	GdkPixbuf        *image;
 	GdkPixbuf        *thumbnail;
-	
+
 	gint              width;
 	gint              height;
 
-	GnomeVFSFileSize  bytes;
+	goffset           bytes;
 	gchar            *file_type;
+	gboolean          threadsafe_format;
 
 	/* Holds EXIF raw data */
-	guchar           *exif_chunk; 
+	guchar           *exif_chunk;
 	guint             exif_chunk_len;
 
 	/* Holds IPTC raw data */
-	guchar           *iptc_chunk; 
+	guchar           *iptc_chunk;
 	guint             iptc_chunk_len;
 
 #ifdef HAVE_EXIF
@@ -53,6 +56,9 @@ struct _EogImagePrivate {
 	gint              orientation;
 	ExifData         *exif;
 #endif
+#ifdef HAVE_EXEMPI
+ 	XmpPtr   xmp;
+#endif
 
 #ifdef HAVE_LCMS
 	cmsHPROFILE       profile;
@@ -71,6 +77,7 @@ struct _EogImagePrivate {
 	GSList           *undo_stack;
 
 	EogTransform     *trans;
+	EogTransform     *trans_autorotate;
 
 	guint             data_ref_count;
 };
